;***************************************************************************
;* Scrolls the map around while the player remains in
;* the center of the screen until an edge is hit, then the
;* player may move independently of scrolling.               
;* 
;*     Coded By: Krylar
;* Last Updated: 7/19/2004
;***************************************************************************

; include the keyboard and map library
Include "keyconstants.bb"
Include "maplib.bb"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Set up the constant vars
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Screen dimension constants
Const ScreenWidth=800,ScreenHeight=600

; setup our widths and heights for loading and collision checking
Const TileWidth = 32         ; width of the map tiles
Const TileHeight = 32         ; height of the map tiles

; determine how many columns/rows to draw per refresh
Const MapColumns = 24       ; how many columns? 
Const MapRows = 15          ; how many rows?

; User can control seeing where the collision boxes are
Global ShowCollisionPoints = 0

; setup the directory references
Global GameDirectory$ = CurrentDir$()
Global GameGraphicsDirectory$ = CurrentDir$() + "\graphics\"

; setup the viewport information
Const ViewPortWidth = MapColumns * TileWidth 
Const ViewPortHeight = MapRows * TileHeight

; set up the default x,y location of where to draw the player.  This should be
; the center of the viewport (or screen if you have no viewport)
Const ScreenX = ViewPortWidth / 2 
Global ScreenY = ViewPortHeight / 2

; use this to keep an unchangeable copy of the center
Const ScreenCenterX = ViewPortWidth / 2
Const ScreenCenterY = ViewPortHeight / 2

; how many pixels to move the player?
; Make sure it divides evenly into TileWidth AND TileHeight
Const MoveDistance = 4        

; Now setup the world coordinates for the player.  
Global WorldX = ScreenX 
Global WorldY = ScreenY 

; And Make the Screen X and Y for the player global as well
Global DrawPlayerX = WorldX
Global DrawPlayerY = WorldY

; initilize the current frame of the player and the current direction
Global CurrenFrame = 0
Global PlayerFacing = 4

; setup our Graphics
Graphics ScreenWidth,ScreenHeight

; set up page flipping
SetBuffer BackBuffer()

; Initialize the Player's bounding boxes
Map_SetupPlayerBoundingBoxes(2,15,30,20,2,30,30,32,4,0,6,32,26,0,28,32)

; Load the tiles for the Map
Map_LoadTiles(GameGraphicsDirectory$ + "alltiles.bmp",TileWidth, TileHeight,0)

; Load the actual map
Map_LoadBinaryMap("scrollmap2.dat")

; Store the actual pixel width/height to a set of globals
Global Map_Pixel_Width = Map_Width * TileWidth 
Global Map_Pixel_Height = Map_Height * TileHeight

; Setup the wall objects so we know where we can expect to collide
SetupWalls()

; Load the player's image
Global Player_Image=LoadAnimImage("graphics\swashbuckler.bmp",TileWidth ,TileHeight,0,32)

; set the appropriate mask so we have transparencies
MaskImage Player_Image,255,0,255

; Setup the X placement for the little title piece
Global TitleX = ScreenWidth/2 - ((Len("Scroll ONLY to the Edges!") * FontWidth()) / 2)  

; lock down the speed of the animation
TimeDelay = CreateTimer(60)

; do a little frame timer to keep the player animation in check
Frame_Timer = MilliSecs()
Map_Timer = MilliSecs()
Animation_Speed = 50     ; adjust this to speed up or slow down the sprite animation
; do this until the player hits ESC
While Not KeyHit(Key_Escape)
 ; clear the screen
 Cls 
  
 ; check to see if any keys are being pressed.
 Direction = CheckKeys()
 ; if not show the standing still frame of the player (based on direction facing)
 If Direction = 0
    If PlayerFacing = 1              ; facing up
       CurrentFrame = 26
    EndIf
    If PlayerFacing = 2              ; facing down
       CurrentFrame = 18
    EndIf
    If PlayerFacing = 3              ; facing left
       CurrentFrame = 0
    EndIf
    If PlayerFacing = 4              ; facing right
       CurrentFrame = 8
    EndIf
 ; if so, change frames accordingly and move the player
 Else
    ; every 50ms update the player's animation frame...this makes it so the animation
    ; is more smooth
    If MilliSecs() > Frame_Timer + Animation_Speed
       ; in our file, frames 0-7 are left and 8-15 are right
       CurrentFrame = CurrentFrame + 1
       If PlayerFacing = 1           ; facing up
          If CurrentFrame < 24
             CurrentFrame = 31
          EndIf
          If CurrentFrame > 31
             CurrentFrame = 24
          EndIf
       EndIf
       If PlayerFacing = 2           ; facing down
          If CurrentFrame < 16
             CurrentFrame = 23
          EndIf
          If CurrentFrame > 23
             CurrentFrame = 16
          EndIf
       EndIf
       If PlayerFacing = 3           ; facing left
          If CurrentFrame > 7
             CurrentFrame = 0
          EndIf
       EndIf
       If PlayerFacing = 4           ; facing right
          If CurrentFrame < 8
             CurrentFrame = 8
          EndIf
          If CurrentFrame > 15
             CurrentFrame = 8
          EndIf
       EndIf
       ; reset the frame timer
       Frame_Timer = MilliSecs()
    EndIf
    ; move the player
 EndIf

 ; move the player and see if we should scroll the map or not
 MoveMap = MovePlayer(Direction,MoveDistance)
 
 ; set our viewport for drawing the map since we want to clip
 ; the map edges on overflow
 Viewport 0,0,ViewPortWidth ,ViewPortHeight 

 ; if the player is standing still
 If Direction = 0
    ; show the map, but don't scroll it
    Map_ShowMap(0,0,0,0,MapColumns,MapRows,TileWidth,TileHeight,ShowCollisionPoints)
 Else
    ; if we want to scroll the map, do that here
    If MoveMap = 1 
       Map_ShowMap(0,0,Direction,MoveDistance,MapColumns,MapRows,TileWidth,TileHeight,ShowCollisionPoints)
    Else
       ; otherwise, just show it
       Map_ShowMap(0,0,0,0,MapColumns,MapRows,TileWidth,TileHeight,ShowCollisionPoints)
    EndIf
 EndIf

 ; Draw the player sprite
 DrawImage(Player_Image,DrawPlayerX ,DrawPlayerY ,CurrentFrame)

 ; if the player wants to see collision info, draw the bounding box here
 If ShowCollisionPoints = 1
    ShowPlayerBoundingBoxes(DrawPlayerX,DrawPlayerY)
 EndIf
    
 ; set to full viewport
 Viewport 0,0,800,600

 ; show some basic information on the player and such
 Text 0,500,"WorldX = " + WorldX + ", WorldY = " + WorldY 
 Text 0,516,"MapWidth = " + Map_Width + ", MapHeight = " + Map_Height
 Text 0,548,"F1 = Toggle Collision Box Visual On/Off (it'll slow down baaaaaaad when on!)"
 Text TitleX,ScreenHeight-16,"Scroll ONLY to the Edges!"

 ; draw a box around the viewport
 Color 255,255,255
 Rect 0,0,ViewPortWidth,ViewPortHeight,0

 ; do the frame-locking delay here
 WaitTimer(TimeDelay)

 ; flip the screen so the user can see what happened
 Flip
   
Wend

; end the program
End



;*************************************************************
; FUNCTION: MovePlayer(...)
; Author: Krylar
; Last Updated: 7/18/2004
;
; This function moves the player's character by a certain
; number of pixels.
;
; Arguments: 
;   Direction: 0=Not Moving, 1=up,2=down,3=left,4=right
;   Distance: how many pixels to move?
;
; Returns:
;   0 = Don't scroll the map
;   1 = Scroll the map
;*************************************************************
Function MovePlayer(Direction,Distance)

 MoveMapX = 0
 MoveMapY = 0
 Select Direction
        ; Standing still
        Case 0
             ; If we're at an edge, let the sprite walk independently of the scroll
             If WorldX <= ScreenX Or WorldX > Map_Pixel_Width - ScreenX
                If WorldX <= ScreenX
                   DrawPlayerX = WorldX
                Else
                   DrawPlayerX = ViewPortWidth - (Map_Width * TileWidth - WorldX)
                EndIf
             Else
                ; We must not be at an edge, so we want to keep the sprite in the middle
                ; and let the program know we're good to scroll
                DrawPlayerX = ScreenX
                MoveMapX = 1
             EndIf

             ; If we're at an edge, let the sprite walk independently of the scroll
             If WorldY <= ScreenY Or WorldY > Map_Pixel_Height - ScreenY
                If WorldY <= ScreenY
                   DrawPlayerY = WorldY
                Else
                   DrawPlayerY = ViewPortHeight - (Map_Height * TileHeight - WorldY)
                EndIf
             Else
                ; We must not be at an edge, so we want to keep the sprite in the middle
                ; and let the program know we're good to scroll
                DrawPlayerY = ScreenY
                MoveMapY = 1
             EndIf

        ; Going Up
        Case 1
             ; save the current World value
             OldY = WorldY
             ; calculate the new value based on the distance
             WorldY = WorldY - Distance 
             ; make sure the new value is >= 0
             If WorldY < 0
                WorldY = 0
             EndIf
             
             ; See if there's a collision
             If Map_CheckWallCollision(WorldX,WorldY) = 1
                ; If so, reset the WorldY value
                WorldY = OldY
             Else
                ; Otherwise, see where we are on the map
                ; If we're at an edge, let the sprite walk independently of the scroll
                If WorldY < ScreenY Or WorldY >= Map_Pixel_Height - ScreenY
                   If WorldY <= ScreenY
                      DrawPlayerY = WorldY
                   Else
                      DrawPlayerY = ViewPortHeight - (Map_Height * TileHeight - WorldY)
                   EndIf
                Else 
                   ; We must not be at an edge, so we want to keep the sprite in the middle
                   ; and let the program know we're good to scroll
                   DrawPlayerY = ScreenY
                   MoveMapY = 1
                EndIf
             EndIf

        ; Going Down
        Case 2
             OldY = WorldY
             WorldY = WorldY + Distance 
             If WorldY > Map_Height * TileHeight - TileHeight
                WorldY = Map_Height * TileHeight - TileHeight
             EndIf
             ; See if there's a collision
             If Map_CheckWallCollision(WorldX,WorldY) = 1
                ; If so, reset the WorldY value
                WorldY = OldY
             Else
                ; Otherwise, see where we are on the map
                ; If we're at an edge, let the sprite walk independently of the scroll
                If WorldY <= ScreenY Or WorldY > Map_Pixel_Height - ScreenY
                   If WorldY <= ScreenY
                      DrawPlayerY = WorldY
                   Else
                      DrawPlayerY = ViewPortHeight - (Map_Height * TileHeight - WorldY)
                   EndIf
                Else
                   ; We must not be at an edge, so we want to keep the sprite in the middle
                   ; and let the program know we're good to scroll
                   DrawPlayerY = ScreenY
                   MoveMapY = 1
                EndIf
             EndIf

        ; Going Left
        Case 3
             OldX = WorldX
             WorldX = WorldX - Distance 
             If WorldX < 0
                WorldX = 0
             EndIf
             ; See if there's a collision
             If Map_CheckWallCollision(WorldX,WorldY) = 1
                ; If so, reset the WorldX value
                WorldX = OldX
             Else
                ; Otherwise, see where we are on the map
                ; If we're at an edge, let the sprite walk independently of the scroll
                If WorldX < ScreenX Or WorldX >= Map_Pixel_Width - ScreenX
                   If WorldX < ScreenX
                      DrawPlayerX = WorldX
                   Else
                      DrawPlayerX = ViewPortWidth - (Map_Width * TileWidth - WorldX)
                   EndIf
                Else
                   ; We must not be at an edge, so we want to keep the sprite in the middle
                   ; and let the program know we're good to scroll
                   DrawPlayerX = ScreenX
                   MoveMapX = 1
                EndIf
             EndIf

        ; Going Right
        Case 4
             OldX = WorldX
             WorldX = WorldX + Distance 
             If WorldX > Map_Width * TileWidth - TileWidth 
                WorldX = Map_Width * TileWidth - TileWidth 
             EndIf
             ; See if there's a collision
             If Map_CheckWallCollision(WorldX,WorldY) = 1
                ; If so, reset the WorldX value
                WorldX = OldX
             Else
                ; Otherwise, see where we are on the map
                ; If we're at an edge, let the sprite walk independently of the scroll
                If WorldX <= ScreenX Or WorldX > Map_Pixel_Width - ScreenX
                   If WorldX <= ScreenX
                      DrawPlayerX = WorldX
                   Else
                      DrawPlayerX = ViewPortWidth - (Map_Width * TileWidth - WorldX)
                   EndIf
                Else
                   ; We must not be at an edge, so we want to keep the sprite in the middle
                   ; and let the program know we're good to scroll
                   DrawPlayerX = ScreenX
                   MoveMapX = 1
                EndIf
             EndIf
 End Select

 ; if either X or Y is scrollable, return 1 to let the caller know
 If MoveMapX = 1 Or MoveMapY = 1
    Return 1
 Else
    Return 0
 EndIf


End Function



;*************************************************************
; FUNCTION: CheckKeys(...)
; Author: Krylar
; Last Updated: 7/19/2004
;
; This function just checks to see which keys are pressed
; and returns the appropriate direction.
;
; Arguments: 
;              N/A
;*************************************************************
Function CheckKeys()

 ; Up
 If KeyDown(Key_ArrowPad_Up) Then
    PlayerFacing = 1  ; make sure player is now facing up
    Return PlayerFacing 
 EndIf

 ; Down
 If KeyDown(Key_ArrowPad_Down) Then
    PlayerFacing = 2  ; make sure player is now facing down
    Return PlayerFacing 
 EndIf

 ; Left
 If KeyDown(Key_ArrowPad_Left) Then
    PlayerFacing = 3  ; make sure player is now facing left
    Return PlayerFacing 
 EndIf

 ; Right
 If KeyDown(Key_ArrowPad_Right) Then
    PlayerFacing = 4  ; make sure player is now facing right
    Return PlayerFacing 
 EndIf

 ; if the player hits F1, toggle visual of collision points
 If KeyHit(Key_F1)
    If ShowCollisionPoints = 0
       ShowCollisionPoints = 1
    Else
       ShowCollisionPoints = 0
    EndIf
 EndIf
 Return 0
End Function


;*************************************************************
; FUNCTION: SetupWalls()
; Author: Krylar
; Last Updated: 7/19/2004
;
; Call the Map_Addwall function to add walls to the collision
; list.
;
; Returns: n/a
;*************************************************************
Function SetupWalls()
         Map_AddWall(0)
         Map_AddWall(1)
         Map_AddWall(2)
         Map_AddWall(3)
         Map_AddWall(5)
         Map_AddWall(6)
         Map_AddWall(7)
         Map_AddWall(8)
         Map_AddWall(10)
         Map_AddWall(11)
         Map_AddWall(12)
         Map_AddWall(13)
         Map_AddWall(15)
         Map_AddWall(16)
         Map_AddWall(17)
         Map_AddWall(18)
         Map_AddWall(20)
         Map_AddWall(21)
         Map_AddWall(22)
         Map_AddWall(23)    
End Function